#import "Controller.h"

#include <time.h>

@implementation Controller


/**************************************************************************
 * ensure that closing the main window ends the program
 *  You must register this class as the application's delegate somewhere:
 *		[NSApp setDelegate:self];
 *		or in Interface Builder, connect the Application ("File's Owner") delegate to this class
 ************************************/
- (BOOL)applicationShouldTerminateAfterLastWindowClosed:
	(NSApplication *)theApplication {
    return YES;
}


/**************************************************************************
 * awakeFromNib
 *	called when interface in unpacked
 *	sets up data
 **************************************************************************/
- (void)awakeFromNib{
	//Get the info for data file path location
	NSBundle *mainBundle = [NSBundle mainBundle];
	NSString *keyboardPath = [mainBundle pathForResource:@"Keyboard Data" ofType:nil];
	NSString *dvorakPath = [mainBundle pathForResource:@"dvorak" ofType:@"keysym"];
	NSString *qwertyPath = [mainBundle pathForResource:@"qwerty" ofType:@"keysym"];
	
	macKeyboard = new Keyboard( [keyboardPath cString] );
	dvorak = new KeyboardLayout( [dvorakPath cString] );
	qwerty = new KeyboardLayout( [qwertyPath cString] );
	dvorakTypist= new Hands( macKeyboard, dvorak );
	qwertyTypist = new Hands( macKeyboard, qwerty );
	
	[dvorakView setKeyboard:macKeyboard layout:dvorak];
	[qwertyView setKeyboard:macKeyboard layout:qwerty];
	
	//Filetypes we can deal with
	fileTypes = [[NSArray alloc] initWithObjects:@"txt", @"text", NSFileTypeForHFSTypeCode('TEXT'), nil];
	NSArray *acceptableDragTypes = [NSArray arrayWithObjects:NSFilenamesPboardType,nil];
	[applicationWindow registerForDraggedTypes:acceptableDragTypes ];

}

/**************************************************************************
 * application:openFile
 *	application's delegate method; drag-and-drop a single file to dock
 **************************************************************************/
- (BOOL)application:(NSApplication *)theApplication openFile:(NSString *)filename{
	NSArray *fileNames = [NSArray arrayWithObject:filename];
	[self application:theApplication openFiles:fileNames];
	return YES;
}

/**************************************************************************
 * application:openFiles
 *	application's delegate method; drag-and-drop multiple files to dock
 **************************************************************************/
- (void)application:(NSApplication *)theApplication openFiles:(NSArray *)filenames{
	[self analyzeFiles:filenames];
}

/**************************************************************************
 * displayDistances
 *	shows the distance figures in the GUI readout
 **************************************************************************/
- (void)displayDistance1:(double)keyboardDistance1 distance2:(double)keyboardDistance2{
	int miles = extractMiles(keyboardDistance1);
	int feet = extractFeetUnderMiles(keyboardDistance1);
	double inches = extractInches(keyboardDistance1);

//cout << "Dvorak: " << miles << " mi. " << feet << " ft. " << inches << " in." << endl;
	
	//First Keyboard First
	if( miles > 0){
		//If the distance has kicked into miles, only display feet and miles
		// (it's silly to go down to inches; the program isn't that precise)
		[[dvorakDistance cellWithTag:0] setIntValue: miles ];
		[[dvorakDistanceLabels cellWithTag:0]
			setStringValue:[NSString stringWithCString:unitNames[MILES][plurality(miles)] ] ];
		
		[[dvorakDistance cellWithTag:1] setFloatingPointFormat:YES left:20 right:0 ];
		[[dvorakDistance cellWithTag:1] setIntValue: feet ];
		[[dvorakDistanceLabels cellWithTag:1]
			setStringValue:[NSString stringWithCString:unitNames[FEET][plurality(feet)] ] ];
	}
	//Otherwise, display feet and inches
	else{
		[[dvorakDistance cellWithTag:0] setIntValue: feet ];
		[[dvorakDistanceLabels cellWithTag:0]
			setStringValue:[NSString stringWithCString:unitNames[FEET][plurality(feet)] ] ];
		
		[[dvorakDistance cellWithTag:1] setFloatingPointFormat:NO left:8 right:2 ];
		[[dvorakDistance cellWithTag:1] setDoubleValue: inches ];
		[[dvorakDistanceLabels cellWithTag:1]
			setStringValue:[NSString stringWithCString:unitNames[INCHES][plurality(inches)] ] ];
		[[qwertyDistanceLabels cellWithTag:1]
			setStringValue:[NSString stringWithCString:unitNames[INCHES][plurality(inches)] ] ];
	}
	
	//Second Keyboard (Qwerty)
	miles = extractMiles(keyboardDistance2);
	feet = extractFeetUnderMiles(keyboardDistance2);
	inches = extractInches(keyboardDistance2);
//cout << "Qwerty: " << miles << " mi. " << feet << " ft. " << inches << " in." << endl;
	if( miles > 0){
		[[qwertyDistance cellWithTag:0] setIntValue: miles ];
		[[qwertyDistanceLabels cellWithTag:0]
			setStringValue:[NSString stringWithCString:unitNames[MILES][plurality(miles)] ] ];
		
		[[qwertyDistance cellWithTag:1] setFloatingPointFormat:YES left:20 right:0 ];
		[[qwertyDistance cellWithTag:1] setIntValue: feet ];
		[[qwertyDistanceLabels cellWithTag:1]
			setStringValue:[NSString stringWithCString:unitNames[FEET][plurality(feet)] ] ];
	}
	//Feet and Inches
	else{
		[[qwertyDistance cellWithTag:0] setIntValue: feet ];
		[[qwertyDistanceLabels cellWithTag:0]
			setStringValue:[NSString stringWithCString:unitNames[FEET][plurality(feet)] ] ];
		
		[[qwertyDistance cellWithTag:1] setFloatingPointFormat:NO left:8 right:2 ];
		[[qwertyDistance cellWithTag:1] setDoubleValue: inches ];
		[[qwertyDistanceLabels cellWithTag:1]
			setStringValue:[NSString stringWithCString:unitNames[INCHES][plurality(inches)] ] ];
	}
	
	//Force Redisplay
	[dvorakDistance displayIfNeeded];
	[dvorakDistanceLabels displayIfNeeded];
	[qwertyDistance displayIfNeeded];
	[qwertyDistanceLabels displayIfNeeded];
}


/**************************************************************************
 * analyzeTextField
 *	takes the text in the text area and calculates its typed distance
 **************************************************************************/
- (IBAction)analyzeTextField:(id)sender{
	NSString *textInput = [textArea string];
	[self typeTextString:textInput];
}


/**************************************************************************
 * typeTextString
 *	takes the text in the text area and calculates its typed distance
 **************************************************************************/
- (void)typeTextString:(NSString *)textInput{
	unsigned int textLength = [textInput length];
	
	dvorakTypist->resetHands();
	qwertyTypist->resetHands();
	[progressDisplay startWithMaximum:textLength]; //primes the progress bar, but does not call it to be displayed

	for(unsigned int counter = 0; counter < textLength; counter++){
		dvorakTypist->typeCharacter( [textInput characterAtIndex:counter] );
		qwertyTypist->typeCharacter( [textInput characterAtIndex:counter] );
		if([progressDisplay updateIfNeededTo:counter]){ //returns true if an update is due
			[self displayDistance1: dvorakTypist->getDistance()
						 distance2: qwertyTypist->getDistance() ];
		}
	}
	dvorakTypist->returnToHomeRow();
	qwertyTypist->returnToHomeRow();

	//Remove Progress Bar
	[progressDisplay endUpdating];

	[self displayDistance1: dvorakTypist->getDistance()
				 distance2: qwertyTypist->getDistance() ];
}

/**************************************************************************
 * openFileWithPanel
 *	prompts the user to select a file to analyze
 **************************************************************************/
- (IBAction)openFileWithPanel:(id)sender{
	NSOpenPanel *openFileDialog = [NSOpenPanel openPanel];
	[openFileDialog setAllowsMultipleSelection:YES];
	[openFileDialog beginSheetForDirectory:nil
					file:nil
					types:fileTypes //which are set up in awakeFromNib
					modalForWindow:applicationWindow
					modalDelegate:self //must be one's own delegate; otherwise, it tries to pass information to the application window
					didEndSelector:@selector(openPanelDidEnd:returnCode:contextInfo:)
					contextInfo:nil
	];
}


/**************************************************************************
 * openPanelDidEnd
 *	handles the task of doing something with the file(s) selected in the open panel
 *  called automatically by the open sheet method
 **************************************************************************/
- (void)openPanelDidEnd:(NSOpenPanel *)openPanel returnCode:(int)returnCode contextInfo:(void *)contextInfo{
	if(returnCode == NSOKButton){
		[openPanel close]; //remove the dialog immediately, because otherwise it will stay onscreen while we evaluate the files
		[self analyzeFiles:[openPanel filenames]];
	}
}


/**************************************************************************
 * draggingEntered
 *	for files being dragged onto the applicationWindow
 **************************************************************************/
- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender{
    NSPasteboard *pasteboard;
    NSDragOperation sourceDragMask;

    sourceDragMask = [sender draggingSourceOperationMask];
    pasteboard = [sender draggingPasteboard];

	if( [[pasteboard types] containsObject:NSFilenamesPboardType]){
		return NSDragOperationCopy;
	}

    return NSDragOperationNone;
}
 
/**************************************************************************
 * performDragOperation
 *	for files being dragged onto the applicationWindow
 **************************************************************************/
- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender{
    NSPasteboard *pasteboard;
    NSDragOperation sourceDragMask;
	BOOL success;

    sourceDragMask = [sender draggingSourceOperationMask];
    pasteboard = [sender draggingPasteboard];

	if( [[pasteboard types] containsObject:NSFilenamesPboardType]){
		//Extract the file paths specified by this drag operation:
		NSArray *files = [pasteboard propertyListForType:NSFilenamesPboardType];
			//we could use the sourceDragMask here to determine something like copy or link, but we're not going to.
		[self analyzeFiles:files];
		success = YES;
	}
	else
		success = NO;

    return success;
	
}

/**************************************************************************
 * analyzeFiles
 *	types the files specified by pathname in filesToOpen
 *  called automatically by the open sheet, openFile, and performDrag methods
 **************************************************************************/
- (void)analyzeFiles:(NSArray *)filesToOpen{
	[textArea setString:@""];
	
	//open a file
	unsigned int counter;
	NSString *amalgamatedString = [NSString string];
	NSString *fileContents;
	
	for(counter = 0; counter < [filesToOpen count]; counter++){
		fileContents = [NSString stringWithContentsOfFile:[filesToOpen objectAtIndex:counter]];
		amalgamatedString = [amalgamatedString stringByAppendingString:fileContents];
	}
	[textArea setString:amalgamatedString];
	[progressDisplay openIndeterminateWithModality:YES]; //start the progress bar twirling before undertaking the potentially *long* task of displaying the file in the text area
	[textArea displayIfNeeded];
	[self typeTextString:amalgamatedString];
}

@end